/*
 * Copyright 2012 Adaptrex, LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.adaptrex.core;

import com.adaptrex.core.persistence.AdaptrexPersistence;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author
 * adaptrex
 */
public class Adaptrex {
    
    private static AdaptrexPersistence defaultPersistence;
    private static Map<String,AdaptrexPersistence> persistenceMap = new HashMap<String,AdaptrexPersistence>();
    
    public static void initPersistence(Class<?> clazz) 
            throws InstantiationException, IllegalAccessException {
        initPersistence(clazz, null);
    }
        
    public static void initPersistence(Class<?> clazz, String factoryName) 
            throws InstantiationException, IllegalAccessException {
        if (factoryName == null || factoryName.isEmpty()) {
            factoryName = "_default";
        }
        
        AdaptrexPersistence persistence = (AdaptrexPersistence) clazz.newInstance();
        if (defaultPersistence == null) {
            defaultPersistence = persistence;
        }
        
        persistenceMap.put(factoryName, persistence);
    }
    
    
    public static AdaptrexPersistence getPersistence() {
        return defaultPersistence;
    }
    
    public static AdaptrexPersistence getPersistence(String factoryName) {
        return persistenceMap.get(factoryName);
    }
    
    public static void destroyPersistence() {
        for (String k : persistenceMap.keySet()) {
            persistenceMap.get(k).shutdown();
        }
    }
}
