/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.adaptrex.core.ext.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.adaptrex.core.ext.rest.ExtConfig;

public class Config {

	/*
	 * General config
	 */
	private String			namespace;
	private String 			tableName;
	private String 			modelName;
	private Class<?> 		clazz;

	/*
	 * Store settings
	 */
	private Integer			start;
	private Integer			pageSize;
	private Boolean			autoSync;
	private Boolean			autoLoad;

	/*
	 * Ext specific settings
	 */
	private Boolean			clearOnPageLoad; 
	private Boolean			clearRemovedOnLoad;
	private Boolean			buffered;
	private Integer			leadingBufferZone;
	private Integer			trailingBufferZone; 
	private Integer			purgePageCount;
	private RestProxy		proxy;

	/*
	 * Sorting and grouping
	 */
	private Boolean			remoteGroup;
	private Boolean			remoteSort;
	private Boolean			remoteFilter;
	private Boolean			sortOnFilter;

	private List<Sorter>	groupers = new ArrayList<Sorter>();
	private List<Sorter>	sorters = new ArrayList<Sorter>();
	private List<Filter> 	filters = new ArrayList<Filter>();

	/*
	 * Response customization
	 */
	private List<String>		associationEntityNames = new ArrayList<String>();
	
	private List<String> 		includes = new ArrayList<String>();
	private List<String> 		excludes = new ArrayList<String>();
	
	private String				where;
	private Map<String,Object> 	params = new HashMap<String,Object>();
	
	
	
	/*
	 * inline is not an ext configuration but indicates if a store
	 * definition include data inline
	 */
	private boolean	inline = false;
	
	/*
	 * This is the name of the persistence unit for
	 * whichever ORM implementation we are using.
	 * 
	 * For example, this is the name of:
	 * 	JPA			: persistence-unit
	 * 	Hibernate	: session-factory
	 */
	private String			persistenceUnitName;
	
	

	protected void group(String property, String direction, String root) {
		groupers.add(new Sorter(property, direction, root));
	}

	protected void sort(String property, String direction, String root) {
		sorters.add(new Sorter(property, direction, root));
	}

	protected void filter(String property, Object value) {
		filters.add(new Filter(property, value));
	}
	
	protected void param(String key, Object value) {
		params.put(key, value);
	}
	
	public void include(String items) {
		for (String item : items.split(",")) {
			includes.add(item);
		}
	}
	
	public void exclude(String items) {
		for (String item : items.split(",")) {
			excludes.add(item);
		}
	}
	
	protected void associations(String associatedModelNames) {
		for (String associatedModelName : associatedModelNames.split(",")) {
			associationEntityNames.add(associatedModelName);
		}
	}

	

	/*********************************************
	 * 
	 * 	Getters and setters
	 * 
	 */
	public String getPersistenceUnitName() {
		return persistenceUnitName;
	}

	public void setPersistenceUnitName(String persistenceUnitName) {
		this.persistenceUnitName = persistenceUnitName;
	}
	
	public String getNamespace() {
		return namespace;
	}
	public void setNamespace(String namespace) {
		this.namespace = namespace;
	}

	public String getTableName() {
		return tableName;
	}
	public void setTableName(String tableName) {
		this.tableName = tableName;
	}
	
	public String getModelName() {
		return modelName;
	}
	public void setModelName(String modelName) {
		this.modelName = modelName;
	}

	public Class<?> getClazz() {
		return clazz;
	}
	public void setClazz(Class<?> clazz) {
		this.clazz = clazz;
	}

	public Integer getPageSize() {
		return pageSize;
	}
	public void setPageSize(Integer pageSize) {
		this.pageSize = pageSize;
	}

	public Boolean getAutoSync() {
		return autoSync;
	}
	public void setAutoSync(Boolean autoSync) {
		this.autoSync = autoSync;
	}

	public Boolean getAutoLoad() {
		return autoLoad;
	}
	public void setAutoLoad(Boolean autoLoad) {
		this.autoLoad = autoLoad;
	}

	public Boolean getClearOnPageLoad() {
		return clearOnPageLoad;
	}
	public void setClearOnPageLoad(Boolean clearOnPageLoad) {
		this.clearOnPageLoad = clearOnPageLoad;
	}

	public Boolean getClearRemovedOnLoad() {
		return clearRemovedOnLoad;
	}
	public void setClearRemovedOnLoad(Boolean clearRemovedOnLoad) {
		this.clearRemovedOnLoad = clearRemovedOnLoad;
	}

	public Boolean getBuffered() {
		return buffered;
	}
	public void setBuffered(Boolean buffered) {
		this.buffered = buffered;
	}

	public Integer getLeadingBufferZone() {
		return leadingBufferZone;
	}
	public void setLeadingBufferZone(Integer leadingBufferZone) {
		this.leadingBufferZone = leadingBufferZone;
	}

	public Integer getTrailingBufferZone() {
		return trailingBufferZone;
	}
	public void setTrailingBufferZone(Integer trailingBufferZone) {
		this.trailingBufferZone = trailingBufferZone;
	}

	public Integer getPurgePageCount() {
		return purgePageCount;
	}
	public void setPurgePageCount(Integer purgePageCount) {
		this.purgePageCount = purgePageCount;
	}
	
	public List<Sorter> getGroupers() {
		return groupers;
	}
	public void setGroupers(List<Sorter> groupers) {
		this.groupers = groupers;
	}
	
	public List<Sorter> getSorters() {
		return sorters;
	}
	public void setSorters(List<Sorter> sorters) {
		this.sorters = sorters;
	}

	public List<String> getAssociationEntityNames() {
		return associationEntityNames;
	}

	public void setAssociationEntityNames(List<String> associationEntityNames) {
		this.associationEntityNames = associationEntityNames;
	}

	public Boolean getRemoteGroup() {
		return remoteGroup;
	}
	public void setRemoteGroup(Boolean remoteGroup) {
		this.remoteGroup = remoteGroup;
	}
	
	public Boolean getRemoteSort() {
		return remoteSort;
	}
	public void setRemoteSort(Boolean remoteSort) {
		this.remoteSort = remoteSort;
	}
	
	public Boolean getRemoteFilter() {
		return remoteFilter;
	}
	public void setRemoteFilter(Boolean remoteFilter) {
		this.remoteFilter = remoteFilter;
	}
	
	public Boolean getSortOnFilter() {
		return sortOnFilter;
	}
	public void setSortOnFilter(Boolean sortOnFilter) {
		this.sortOnFilter = sortOnFilter;
	}

	public List<String> getIncludes() {
		return includes;
	}
	public void setIncludes(List<String> includes) {
		this.includes = includes;
	}
	
	public List<String> getExcludes() {
		return excludes;
	}
	public void setExcludes(List<String> excludes) {
		this.excludes = excludes;
	}

	public RestProxy getProxy() {
		return proxy;
	}
	public void setProxy(RestProxy proxy) {
		this.proxy = proxy;
	}

	public String getWhere() {
		return where;
	}
	public void setWhere(String where) {
		this.where = where;
	}

	public Map<String,Object> getParams() {
		return params;
	}
	public void setParams(Map<String,Object> params) {
		this.params = params;
	}

	public List<Filter> getFilters() {
		return filters;
	}
	public void setFilters(List<Filter> filters) {
		this.filters = filters;
	}
	
	public Integer getStart() {
		return start;
	}
	public void setStart(Integer start) {
		this.start = start;
	}

	
	public boolean isInline() {
		return inline;
	}

	public void setInline(boolean inline) {
		this.inline = inline;
	}

	
	
	/*
	 * Apply Ext Config
	 */
	protected void applyExtConfig(ExtConfig options) {
		if (options.getLimit() != null) setPageSize(options.getLimit());
		if (options.getStart() != null) setStart(options.getStart());
		if (options.getSort() != null) {
			for (Map<String,String> sortParam : options.getSort()) {
				sort(sortParam.get("property"), sortParam.get("direction"), sortParam.get("root"));
			}
		}
		if (options.getFilter() != null) {
			for (Map<String,String> filterParam : options.getFilter()) {
				filter(filterParam.get("property"), filterParam.get("value"));
			}
		}		
		
		if (options.getInclude() != null) include(options.getInclude());
		if (options.getExclude() != null) exclude(options.getExclude());
		if (options.getAssociations() != null) associations(options.getAssociations());
	}
}
