/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.adaptrex.core.ext.data;

import com.adaptrex.core.Adaptrex;
import com.adaptrex.core.persistence.AdaptrexPersistence;
import com.fasterxml.jackson.annotation.JsonInclude;

public class FieldDefinition {

	private String		name;
	private String		type;
	private Boolean		useNull;
	private String		dateFormat;
	
	private static AdaptrexPersistence persistence = Adaptrex.getPersistence();
	
	public FieldDefinition(String name, String type, boolean useNull) {
		this.name = name;
		this.type = type;
		this.useNull = useNull;
	}
	
	public FieldDefinition(java.lang.reflect.Field field) {
		/*
		 * Get the field name
		 */
		this.name = field.getName();

		
		/*
		 * Get the type.  If it's a join column set it to auto
		 */
		String columnType = field.getType().getSimpleName().toLowerCase();
		
		if (columnType.equals("integer") || columnType.equals("long")) {
			columnType = "int";
		}

		else if (columnType.equals("double")) {
			columnType = "float";
		}
		
		else if (columnType.equals("time")) {
			columnType = "date";
			this.dateFormat = "H:i:s";
		}
		
		else if (columnType.equals("calendar") || columnType.equals("date")) {
			columnType = "date";
			this.dateFormat = "Y-m-d";
		}

		this.type = columnType;
		
		/*
		 * Ext always wants "null" for number fields otherwise it generates a 0
		 */
		if (this.type.equals("int") || this.type.equals("float")) this.useNull = true;
	}
	
	
	
	
	@JsonInclude
	public String getName() {
		return this.name;
	}
	
	@JsonInclude
	public String getType() {
		return this.type;
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean getUseNull() {
		return this.useNull;
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getDateFormat() {
		return this.dateFormat;
	}
}
