/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.adaptrex.core.persistence.jpa;

import org.eclipse.persistence.logging.AbstractSessionLog;
import org.eclipse.persistence.logging.SessionLog;
import org.eclipse.persistence.logging.SessionLogEntry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
 
/*
 * EclipseLink implements it's own logging instead of one of the standard loggers.
 * Take it back using this class by configuring it in your persistence.xml
 * 
 * <property name="eclipselink.logging.logger" value="com.adaptrex.core.persistence.jpa.EclipseLinkSlf4jLogger"/>
 */
public class EclipseLinkSlf4jLogger extends AbstractSessionLog implements SessionLog {
 
    public static final Logger log = LoggerFactory.getLogger(EclipseLinkSlf4jLogger.class);
 
    @Override
    public void log(SessionLogEntry sle) {
    	 String msg = sle.getMessage();
    	
    	 switch (sle.getLevel()) {
    	 	case SEVERE	: log.error(msg); break;
	        case WARNING: log.warn(msg); break;
	        case INFO	: log.info(msg); break;
	        default		: log.debug(msg); break;
    	 }
	        
        if (sle.hasException()) {
        	log.error("Exception in EclipseLink", sle.getException());
        }
    }
}
